//
// T-Plan Robot Enterprise, automated testing tool.
// Copyright (C) 2009-2016 T-Plan Limited (http://www.t-plan.com),
// Cloisters House, 8 Battersea Park Road, London, SW8 4BG, United Kingdom.
// All rights reserved.
//
var tplanAppName = "com.t-plan.TPlanApp";
var target = UIATarget.localTarget();
var tplan = target.frontMostApp().bundleID() == tplanAppName;
var host = target.host();
var file = "INFILE";
var lastSeqNo = -1;
var exitFlag = 0;

// Screen width and height
var w = target.rect().size.width;
var h = target.rect().size.height;

// Variable used to join text from a sequence of 'type' commands 
var textToType = "";
var lastKeyboardSeqNo = -1;

var keyShiftNames = ["shift"];
var keySpaceNames = ["Mezerník", "space"];
var keyNextNames = ["Next keyboard"];
var keyReturnNames = ["Go"];
var keyDeleteNames = [ "delete"];
var keyAmpersandNames = [ "ampersand" ];
var keyMoreNames = [ "more, numbers" ];
var keyHideNames = [ "Hide keyboard" ];
var keyLeftArrowNames = [ "Left Arrow" ];
var keyRightArrowNames = [ "Right Arrow" ];
var keyUndoNames = [ "Undo" ];
var keyDictateNames = [ "Dictate" ];
var keyEmojiNames = [ "Emoji keyboard" ];

function type() {
    var text = textToType;
    if (text.length > 0) {
        textToType = "";
        
        if (!isKeyboard()) {
            return;
        }
        
        // Iterate for all characters in the typed text
        for (var i = 0, len = text.length; i < len; i++) {            
            var c = String(text[i]);
            // "\\" is back slash
            // "\XXXX" is character 0xXXXX (escaped unicode)
            if (c == "\\") {
                var c2 = text[++i];
                if (c2 != "\\") { // Escaped UTF-8 sequence
                    c2 = String(c2) + String(text[i+1]) + String(text[i+2]) + String(text[i+3]);
                    i += 3;
                    var code = parseInt(c2, 16);
                    
                    switch (code) {
                        case 0x20: // Space
                            tapSpace();
                            return;
                        case 0x08: // Back space
                        case 0x7f: // Delete
                            tapDelete();
                            return;
                        case 0x0a: // Line feed (Enter) -> press Go/Return
                            tapReturn();
                            return;
                        case 0x09: // Tab
                            tapNext();
                            return;
                        case 0x26: // Ampersand
                            c = keyAmpersandNames;
                            break;
                        case 0x1b: // Escape
                            c = keyHideNames;
                            break;
                        case 0x25: // Left
                            c = keyLeftArrowNames;
                            break;
                        case 0x27: // Right
                            c = keyRightArrowNames;
                            break;
                        case 0xffcb: // Undo
                            c = keyUndoNames;
                            break;
                        case 0x71: // F2
                            c = keyDictateNames;
                            break;
                        case 0x72: // F3
                            c = keyEmojiNames;
                            break;
                        case 0xffc8: // F11
                            target.clickVolumeDown();
                            return;
                        case 0xffc9: // F12
                            target.clickVolumeUp();
                            return;
                        default:
                            UIALogger.logDebug("PREFIX71Unsupported character: "+code+" (0x"+c2+")");
                            return;
                    }
                }
            }
            
            // Try on the current keyboard
            var found = tapKey(c);
            if (!found) {
                tapByName(keyShiftNames);
                found = tapKey(c);
            }
            if (!found) {
                tapMore();
                found = tapKey(c);
                if (!found) {
                    tapByName(keyShiftNames);
                    found = tapKey(c);
                }
            }
            if (!found) {
                UIALogger.logDebug("key not found: "+c);
            }
        }
        UIALogger.logDebug("PREFIX21"+lastKeyboardSeqNo);
    }
}

function isKeyboard() {
    var app = target.frontMostApp();
    var id = app.bundleID();
    if (id == "com.apple.springboard" || id == "com.t-plan.TPlanApp") {
        // Springboard -> we are in the Home screen
        return false;
    }
    return true;
}

function tapByName(s) {
    var keys = target.frontMostApp().keyboard().elements();    
    var len = keys.length;
    for (var i = 0; i < len; i++) {
        var key = keys[i];
        var keyName = key.name();
        if (keyName) {
            if (s instanceof Array) {
                for (var j=0; j<s.length; j++) {
                    if (keyName.toLowerCase().startsWith(s[j].toLowerCase())) {
                        UIATarget.localTarget().tap(key.rect());
                        return true;
                    }
                }
            } else {
                if (keyName.toLowerCase().startsWith(s)) {
                    UIATarget.localTarget().tap(key.rect());
                    return true;
                }
            }
        }
    }
    return false;
}

function tapKey(c) {
    //    UIALogger.logDebug("tapKey("+c+")");
    //    target.frontMostApp().keyboard().logElementTree();
    var keyboard = target.frontMostApp().keyboard();
    var keys = keyboard.elements();    
    var klen = keys.length;
    
    var found = false;
       
    // Iterate for all the elements of the current keyboard
    for (var k = 0; k < klen; k++) {
        var key = keys[k];
        var keyName = key.name();
        if (c instanceof Array) {
            for (var j=0; j<c.length; j++) {
                if (keyName == c[j]) {
                    tapButton(key);
                    found = true;
                    break;
                } 
            } 
        } else if (keyName == c) {
            tapButton(key);
            found = true;
            break;
        }           
    }
    return found;
}

function tapButton(button) {
    button.logElementTree();
    target.tap(button.rect());
}

// Find the Space key as the largest (widest) UIAKey in the last row of keys
function tapSpace() {
    var found = tapByName(keySpaceNames);
    if (found) {
        return;
    }
    var keyboard = target.frontMostApp().keyboard();
    var keys = keyboard.elements();    
    var klen = keys.length;
    var largest;
    var lr;
    
    for (var k = klen-1; k >= 0; k--) {
        var key = keys[k];
        if (key instanceof UIAKey) {
            if (!largest) {  // No stored key (first loop)
                largest = key;
                lr = largest.rect();
            } else {
                var kr = key.rect();
                if (kr.origin.y + kr.size.height < lr.origin.y) { // Not the last row of keys
                    break;            
                } else if (lr.size.width < kr.size.width) {  // Larger than the stored one
                    largest = key;
                    lr = largest.rect();
                }
            }
        }
    }
    tapButton(largest);
}

// Find Enter as the last UIAButton in the last row when going from the right to
// the left before there is any UIAKey instance
function tapReturn() {
    var found = tapByName(keyReturnNames);
    if (found) {
        return;
    }
    var keyboard = target.frontMostApp().keyboard();
    var keys = keyboard.elements();    
    var k = keys.length;
    var candidate;
    // Can not test "instanceof UIAButton" because it seems to be the superclass of UIAKey 
    while (--k >= 0 && !(keys[k] instanceof UIAKey)) {
        candidate = keys[k];
    } 
    tapButton(candidate);
}

// Find Delete as the first UIAKey button from the right in the second row with 
// the name longer than one char 
function tapDelete() {
    var found = tapByName(keyDeleteNames);
    if (found) {
        return;
    }
    var keyboard = target.frontMostApp().keyboard();
    var keys = keyboard.elements();  
    var key;
    var range = getKeyRange(keys, 1);
    for (var i=range[1]; i >= range[0]; i--) {
        key = keys[i];
        if (key instanceof UIAKey && key.name().length > 1) {
            tapButton(key);
            return;
        }
    }
}

// Next is the very first UIAButton instance in the last row
function tapNext() {
    var found = tapByName(keyNextNames);
    if (found) {
        return;
    }
    var keyboard = target.frontMostApp().keyboard();
    var keys = keyboard.elements();  
    var key;
    var range = getKeyRange(keys, 0);
    
    for (var i = range[0]; i <= range[1]; i++) {
        key = keys[i];
        if (!(key instanceof UIAKey)) {
            tapButton(key);
            return;
        }
    }    
}

// Next is the very first UIAKey instance in the last row
function tapMore() {
    var found = tapByName(keyMoreNames);
    if (found) {
        return;
    }
    var keyboard = target.frontMostApp().keyboard();
    var keys = keyboard.elements();  
    var key;
    var range = getKeyRange(keys, 0);
    
    for (var i = range[0]; i <= range[1]; i++) {
        key = keys[i];
        if (key instanceof UIAKey) {
            tapButton(key);
            return;
        }
    }    
}

function getKeyRange(keys, rowIndex) {
    var is = [0, keys.length-1, 0];
    var kr;
    var pr;
    var row = 0;
    
    for (var i = keys.length - 1; i >= 0; i--) {
        kr = keys[i].rect();
        if (pr && kr.origin.y + kr.size.height < pr.origin.y) {  // New row
            row++;
            if (row == rowIndex) {
                is[1] = i;
            }
        }
        if (row == rowIndex) {
            is[0] = i;
        }
        is[2] = row + 1;
        pr = kr;
    }
    return is;
}

if (tplan) {
    var home = false;
    do {
        if (target.frontMostApp().bundleID() == "com.apple.springboard") {
            home = true;
            break;
        } 
    } while (!home);
}

UIALogger.logDebug("PREFIX00" + target.model() + "SEPARATOR" + target.systemVersion() 
    + "SEPARATOR" + w + "SEPARATOR" + h + "SEPARATOR" + tplanAppName);
UIALogger.logDebug("PREFIX01");

do {
    var result = host.performTaskWithPathArgumentsTimeout("/bin/cat", [file], 5);
    if (result.exitCode != 0) {
        UIALogger.logDebug("PREFIX70Failed to cat "+file);
        continue;
    }
    var out = result.stdout.toString().trim();
    if (out.length > 0) {
        var lines = out.split('\n');
        
        // Read the seq No of the last command
        var trailingSeqNo = -1;
        for (var i = lines.length-1; i >= 0; i--) {
            trailingSeqNo = parseInt(lines[lines.length - 1].split(" ")[0]);
            if (isNaN(trailingSeqNo)) {
                trailingSeqNo = -1;
            } else {
                break;
            }
        }        
                
        for (i = 0; i < lines.length; i++) {
            //UIALogger.logDebug("Processing "+lines[i]);
            var slices = lines[i].split(" ");
            var no = parseInt(slices[0]);
            if (no < 0) {  // Negative No means seq number reset
                lastSeqNo = -1;
                continue;
            }
            if (no > lastSeqNo) {
                lastSeqNo = no;
                
                // There's at least one command to process -> log the trailing seq no
                // to make the Robot process know that we are processing the commands
                if (trailingSeqNo >= 0) {
                    UIALogger.logDebug("PREFIX20"+trailingSeqNo);
                    trailingSeqNo = -1;
                }
                
                // Execute the command
                var cmd = slices[1];
                var fullCmd = lines[i].substr(slices[0].length + 1);
                //UIALogger.logDebug("Executing command #"+no+": '"+fullCmd+"'");

                if (cmd != "type") {
                    type();
                }

                // Command interpret starts here ==============================
                if (cmd == "tap") {  // tap <x> <y> <cnt>                    
                    if (slices.length != 5) {
                        UIALogger.logDebug("PREFIX71Command syntax error: expecting 3 tap arguments in "+fullCmd);
                    } else {
                        var x = parseInt(slices[2]);
                        var y = parseInt(slices[3]);
                        var cnt = parseInt(slices[4]);
                        target.tap({
                            x:x, 
                            y:y
                        });
                    }
                    UIALogger.logDebug("PREFIX21"+no);
                } else if (cmd == "swipe") {  // swipe <src_x> <src_y> <tgt_x> <tgt_y>
                    if (slices.length != 6) {
                        UIALogger.logDebug("PREFIX71Command syntax error: expecting 4 swipe arguments in "+fullCmd);
                    } else {
                        x = parseInt(slices[2]);
                        y = parseInt(slices[3]);
                        var tx = parseInt(slices[4]);
                        var ty = parseInt(slices[5]);
                        target.flickFromTo({
                            x:x, 
                            y:y
                        }, {
                            x:tx, 
                            y:ty
                        });
                    }
                    UIALogger.logDebug("PREFIX21"+no);
                } else if (cmd == "zoom") {  // pinch/zoom <0|1> <start_x> <start_y> <end_x> <end_y> <duration_secs>
                    if (slices.length != 8) {
                        UIALogger.logDebug("PREFIX71Command syntax error: expecting 6 zoom arguments in "+fullCmd);
                    } else {
                        var open = parseInt(slices[2]);
                        x = parseInt(slices[3]);
                        y = parseInt(slices[4]);
                        var ex = parseInt(slices[5]);
                        var ey = parseInt(slices[6]);
                        var duration = parseFloat(slices[7]);
                        if (open == 0) {
                            target.pinchCloseFromToForDuration({
                                x:x, 
                                y:y
                            }, {
                                x:ex, 
                                y:ey
                            }, duration);
                        } else {
                            target.pinchOpenFromToForDuration({
                                x:x, 
                                y:y
                            }, {
                                x:ex, 
                                y:ey
                            }, duration);
                        }
                    }
                    UIALogger.logDebug("PREFIX21"+no);
                } else if (cmd == "press") {  // Functional key mappings
                    if (slices.length != 3) {
                        UIALogger.logDebug("PREFIX71Command syntax error: expecting 1 type argument in "+fullCmd);
                    } else {
                        var code = parseInt(slices[2]);
                    // TODO - volume keys, shake, rotate gesture etc.
                    }
                    UIALogger.logDebug("PREFIX21"+no);
                } else if (cmd == "type") {  // type <text>
                    if (slices.length != 3) {
                        UIALogger.logDebug("PREFIX71Command syntax error: expecting 1 type argument in "+fullCmd);
                    } else {
                        textToType += slices[2];
                        lastKeyboardSeqNo = no;
                    }
                } else if (cmd == "exit") {  // exit
                    type();
                    exitFlag = 1;
                    UIALogger.logDebug("PREFIX21"+no);
                    break;
                } else if (cmd == "dumpkeys") {
                    UIALogger.logDebug("***************************");
                    UIALogger.logDebug(" KEYBOARD DUMP");
                    UIALogger.logDebug("***************************");
                    var keyboard = target.frontMostApp().keyboard();
                    var keys = keyboard.elements();    
                    keyboard.logElementTree();
                    UIALogger.logDebug("***************************");
                    UIALogger.logDebug(" KEYBOARD SUMMARY");
                    UIALogger.logDebug("***************************");
                    var is = getKeyRange(keys, 0);
                    var rowCount = is[2];
                    UIALogger.logDebug(" There are "+rowCount+" rows of keys as follows:");
                    for (var i=0; i<rowCount; i++) {
                        is = getKeyRange(keys, i);
                        UIALogger.logDebug(" Row #"+i+" ["+is[0]+","+is[1]+"] has "+(is[1]-is[0]+1)+" keys from '"
                            +keys[is[0]].name()+"' to '"+keys[is[1]].name()+"'");
                    }                    
                    UIALogger.logDebug("***************************");
                }
            }
        }
        type();
    }
} while (exitFlag == 0);

UIALogger.logDebug("PREFIX90");