#!/bin/bash

# Switch to the script directory if needed.
# We have to do it because the Java command references the libs as relative
if [ "$0" != './robot.sh' ]; then
  dir=`echo "$0" | sed 's/robot\.sh//'`
  cd "$dir"
fi
dir=`pwd`

UFILE="$dir/update.sh"
if [[ -f "$UFILE" ]]; then
  chmod a+x "$UFILE"
  "$UFILE"
  rm "$UFILE"
fi

# On Mac OS X we inject the dock app name.
# Make sure to keep the more general start command first 
# because this script is parsed by the RAP agent for the first Java command
os=`uname -s`
arch=`uname -m`
JHOME=

if [ "$os" != "Darwin" ]; then
   # We are on Linux/Unix
   # Look for Java installed in the Robot folder. Since there's no Temurin x32 
   # for Linux we only support x64
   J64=$dir/java/linux-amd64/bin
   if [[ -d "$J64" ]]; then
      JHOME="$J64/"
      echo "Using Java from: $JHOME"
   fi
   "${JHOME}java" -Dsun.java2d.xrender=false -Xmx512m -Djava.system.class.loader=com.tplan.robot.util.LazyClassLoader -classpath "./*:libs/*:plugins/*:libs/selenium/*:libs/poi/*" com.tplan.robot.ApplicationSupport $*

else
   # We are on Mac OS X
   # Look for Java installed in the Robot folder
   J64=$dir/java/mac-x86_64/Contents/Home/bin
   J64M1=$dir/java/mac-aarch64/Contents/Home/bin

   # The classic x84_64 Java can run on any architecture
   if [[ -d "$J64" ]]; then
      JHOME="$J64/"
      echo "Using Java from: $JHOME"
   fi
   if [ "$arch" == "arm64" ]; then
      # Look if there's an M1 Java
      if [[ -d "$J64M1" ]]; then
          JHOME="$J64M1/"
          echo "Using Java from: $JHOME"
      fi
   fi

   # Mac OS X requires special flags to enable Chromium on Java 16+
   # We add these flags for any Java 9+
   JV=`${JHOME}java -version 2>&1 | head -n 1 | cut -d " " -f 3`
   if grep -q "1.8.0" <<< "$JV"; then
     # We are on Java 8 -----------------
     "${JHOME}java" -Dsun.java2d.xrender=false -Djava.system.class.loader=com.tplan.robot.util.LazyClassLoader -Xmx512m "-Xdock:name=HeartCore Robo Desktop" -classpath "./*:libs/*:plugins/*:libs/selenium/*:libs/poi/*" com.tplan.robot.ApplicationSupport $* 
   else
     # We are on Java 9+ -----------------
     "${JHOME}java" -Dsun.java2d.xrender=false -Djava.system.class.loader=com.tplan.robot.util.LazyClassLoader -Xmx512m "-Xdock:name=HeartCore Robo Desktop" --add-opens java.desktop/sun.awt=ALL-UNNAMED --add-opens java.desktop/sun.lwawt=ALL-UNNAMED --add-opens java.desktop/sun.lwawt.macosx=ALL-UNNAMED -classpath "./*:libs/*:plugins/*:libs/selenium/*:libs/poi/*" com.tplan.robot.ApplicationSupport $*
   fi
fi
